--[[

--Clase PADRE, del cual heredaran las clases de naves.

]]

-- cargar las librerias externas
local LG, LA, LK, LM = love.graphics, love.audio, love.keyboard, love.mouse
local PHYS   = require("extlibs/windfield")


-- definir la clase jugador
local Jugador = require("extlibs/30log")("Jugador", {

    

    -- audios
    inactivoAudio            = LA.newSource("src/jugador/player_idle.ogg", "static"),
    --avanzarAudio           = LA.newSource(""), --TODO: audio
    --dispararAudio          = LA.newSource(""), --TODO: audio
    --morirAudio             = LA.newSource(""), --TODO: audio
    
    -- caracteristicas
    claseDeColision          = "jugador",
    
    -- velocidad linear
    velocidadDeMovimiento    = 1,
    multiplicadorDeVelocidad = 1000,
    
    -- velocidad angular 
    velocidadDeRotacion      = 1,
    multiplicadorDeRotacion  = 500000,
    
    -- frenado linear
    velocidadDeFrenadoL      = 1 ,
    divisorDeFrenadoL        = 100,
    
    -- frenado angular
    velocidadDeFrenadoA      = 1,  -- the A means angular
    multiplicadorDeFrenadoA  = 1000,  -- same here
    
    -- caracteristicas fisicas
    cuerpo                   = nil,
    masaAdecuada             = 5,
    
    -- caracteristicas del sprite 
    sprite                   = nil,
    spriteAngulo             = -90     --[[en grados por favor]]
  })


function Jugador:moverConTeclado()
  
  if self.cuerpo then
    -- establece la masa en uno
    if self.cuerpo:getMass() > self.masaAdecuada then self.cuerpo:setMass(self.masaAdecuada) end
    
    -- establecer frenado
    self.cuerpo:setLinearDamping (self.velocidadDeFrenadoL)
    self.cuerpo:setAngularDamping(self.velocidadDeFrenadoA)
    
    --[[ Con las teclas W,A,S,D ]]
    --{{
    local function aplicarFuerza()
      return self.velocidadDeMovimiento * self.multiplicadorDeVelocidad
    end
    
    if LK.isDown("w") then self.cuerpo:applyForce(0, -aplicarFuerza()) end
    if LK.isDown("s") then self.cuerpo:applyForce(0,  aplicarFuerza()) end
    
    if LK.isDown("a") then self.cuerpo:applyForce(-aplicarFuerza(), 0) end
    if LK.isDown("d") then self.cuerpo:applyForce( aplicarFuerza(), 0) end
    --}} 
  else
    
  end
end

-- el sprite de la nave apuntara hacia la posicion del mouse
function Jugador:rotacionDelSpriteHaciaMouse()
  local velocidadRotacion = self.velocidadDeRotacion * self.multiplicadorDeRotacion
  
  local cuerpoX = self.cuerpo:getX()
  local cuerpoY = self.cuerpo:getY()
  
  local anguloCuerpo = math.deg(self.cuerpo:getAngle())
  
  -- LM means love.mouse 
  local anguloMouse  = math.deg(math.atan2(LM.getY() - cuerpoY, LM.getX() - cuerpoX))
  
  if anguloMouse > anguloCuerpo then
    self.cuerpo:applyTorque(velocidadRotacion)
  else
    self.cuerpo:applyTorque(-velocidadRotacion)
  end
  
end



-- funcion que dibuja el sprite, efectos, proyectiles
function Jugador:dibujar()
  
  
  if self.sprite then
    LG.setColor(0.3,1,0,1)
    local escalaX   =  self.cuerpo:getRadius() / self.sprite:getWidth()  * 2
    local escalaY   =  self.cuerpo:getRadius() / self.sprite:getHeight() * 2
    
    LG.draw(self.sprite, 
      self.cuerpo:getX()   ,     -- x
      self.cuerpo:getY()   ,     -- y
      self.cuerpo:getAngle() + math.rad(90), -- angulo
      escalaX,                   
      escalaY,
      self.sprite:getWidth() / 2,
      self.sprite:getHeight() / 2)
  end
end

-- una sola funcion para llamar las demas de la clase
function Jugador:actualizar()
  self:rotacionDelSpriteHaciaMouse()
  self:moverConTeclado()
end

return Jugador